﻿// --------------------------------
// <copyright file="BlockBuilder.cs" company="COMPANY">
//     Copyright (C) COMPANY.  All rights reserved.
// </copyright>
// --------------------------------

using UnityEngine;

/// <summary>
/// BlockBuilder class.
/// </summary>
public class BlockBuilder : MonoBehaviour
{
	/// <summary>
	/// Building prefab to copy.
	/// </summary>
	public GameObject buildingPrefab;

	/// <summary>
	/// Population manager, set by parent.
	/// </summary>
	public PopulationManager populationManager;

	public float bigBuilding = 2f;
	public float smallBuilding = 0.9f;
	public float offset = 0.55f;
	public float skyscraperRate = 0.05f;

    /// <summary>
    /// Initialize script state.
    /// </summary>
    internal void Start()
    {
		var buildingCount = Random.Range(1, 5);

		switch (buildingCount)
		{
			case 1:
				this.InstantiateBuilding(Locale.Center);

				break;
			case 2:
				if (Random.value < 0.5f)
				{
					this.InstantiateBuilding(Locale.East);
					this.InstantiateBuilding(Locale.West);
				} 
				else
				{
					this.InstantiateBuilding(Locale.North);
					this.InstantiateBuilding(Locale.South);
				}

				break;
			case 3:
				switch (Random.Range(0, 4))
				{
					case 0:
						this.InstantiateBuilding(Locale.SouthEast);
						this.InstantiateBuilding(Locale.SouthWest);
						this.InstantiateBuilding(Locale.North);
						break;
					case 1:
						this.InstantiateBuilding(Locale.NorthWest);
						this.InstantiateBuilding(Locale.SouthWest);
						this.InstantiateBuilding(Locale.East);
						break;
					case 2:
						this.InstantiateBuilding(Locale.NorthEast);
						this.InstantiateBuilding(Locale.NorthWest);
						this.InstantiateBuilding(Locale.South);
						break;
					case 3:
						this.InstantiateBuilding(Locale.NorthEast);
						this.InstantiateBuilding(Locale.SouthEast);
						this.InstantiateBuilding(Locale.West);
						break;
				}

				break;
			case 4:
				this.InstantiateBuilding(Locale.NorthEast);
				this.InstantiateBuilding(Locale.SouthEast);
				this.InstantiateBuilding(Locale.SouthWest);
				this.InstantiateBuilding(Locale.NorthWest);

				break;
		}
    }

	private void InstantiateBuilding(Locale locale)
	{
		var isNotSkyscraper = Random.value > this.skyscraperRate;

		// height between 0.5 and 2.5, or 4.0 and 5.0
		var height = Constants.floorHeight * Random.Range(isNotSkyscraper ? 1 : 8, isNotSkyscraper ? 6 : 11);

		var building = (GameObject)Instantiate(this.buildingPrefab);
		building.transform.parent = this.transform;
		building.name = string.Format("Building ({0})", locale);
		building.GetComponent<Building>().populationManager = this.populationManager;

		var size = new Vector2(this.bigBuilding, this.bigBuilding);
		var location = Vector2.zero;

		switch (locale)
		{
			case Locale.North:
				size = new Vector2(this.bigBuilding, this.smallBuilding);
				location = new Vector2(0f, this.offset);
				break;
			case Locale.East:
				size = new Vector2(this.smallBuilding, this.bigBuilding);
				location = new Vector2(this.offset, 0f);
				break;
			case Locale.South:
				size = new Vector2(this.bigBuilding, this.smallBuilding);
				location = new Vector2(0f, -this.offset);
				break;
			case Locale.West:
				size = new Vector2(this.smallBuilding, this.bigBuilding);
				location = new Vector2(-this.offset, 0f);
				break;
			case Locale.NorthEast:
				size = new Vector2(this.smallBuilding, this.smallBuilding);
				location = new Vector2(this.offset, this.offset);
				break;
			case Locale.SouthEast:
				size = new Vector2(this.smallBuilding, this.smallBuilding);
				location = new Vector2(this.offset, -this.offset);
				break;
			case Locale.SouthWest:
				size = new Vector2(this.smallBuilding, this.smallBuilding);
				location = new Vector2(-this.offset, -this.offset);
				break;
			case Locale.NorthWest:
				size = new Vector2(this.smallBuilding, this.smallBuilding);
				location = new Vector2(-this.offset, this.offset);
				break;
		}

		building.transform.localPosition = new Vector3(location.x, (height * Constants.floorHeight) + 0f, location.y);
		building.transform.localScale = new Vector3(size.x, height, size.y);
	}
	
    /// <summary>
    /// Locale enum.
    /// </summary>
	private enum Locale
	{
		Center,
		North,
		NorthEast,
		East,
		SouthEast,
		South,
		SouthWest,
		West,
		NorthWest,
	}
}