﻿// --------------------------------
// <copyright file="DisasterManager.cs" company="COMPANY">
//     Copyright (C) COMPANY.  All rights reserved.
// </copyright>
// --------------------------------

using System;
using System.Collections;
using UnityEngine;

/// <summary>
/// DisasterManager class.
/// </summary>
public class DisasterManager : MonoBehaviour
{
	/// <summary>
	/// Tornado prefab to copy.
	/// </summary>
	public GameObject tornadoPrefab;

	/// <summary>
	/// Meteor prefab to copy.
	/// </summary>
	public GameObject meteorPrefab;

	/// <summary>
	/// Earthquake prefab to copy.
	/// </summary>
	public GameObject earthquakePrefab;

	// TODO: state manager
	public PopulationManager populationManager;
	public int casualtyTarget = 500;
	public UILabel headlineLabel;
	public GameObject continueButton;

    /// <summary>
    /// Input handler.
    /// </summary>
    public InputManager input;

    /// <summary>
    /// Particle effect manager;
    /// </summary>
    public ParticleEffectManager particleEffectManager;

    /// <summary>
    /// Selected disaster, if any.
    /// </summary>
    public DisasterType selectedDisaster;
	
	/// <summary>
	/// Max number of turns.
	/// </summary>
	public int disasterCount = 3;

    /// <summary>
    /// Initialize script state.
    /// </summary>
    internal void Start()
    {

    }

    /// <summary>
    /// Called when script is enabled.
    /// </summary>
    internal void OnEnable()
    {
        this.input.PickEvent += this.HandlePickEvent;
    }

    /// <summary>
    /// Called when script is disabled.
    /// </summary>
    internal void OnDisable()
    {
        this.input.PickEvent -= this.HandlePickEvent;
    }

    private void HandlePickEvent(Vector3 point)
    {
		if (this.disasterCount <= 0)
		{
			return;
		}

        switch (this.selectedDisaster)
        {
            case DisasterType.Lightning:
                this.particleEffectManager.InstantiateParticles(this.particleEffectManager.lightningPrefab, point);
                break;
            case DisasterType.Tornado:
				Instantiate(this.tornadoPrefab, new Vector3(point.x, 0f, point.z), this.tornadoPrefab.transform.rotation); 
				break;
            case DisasterType.Meteor:
				Instantiate(this.meteorPrefab, new Vector3(point.x, this.meteorPrefab.transform.position.y, point.z), this.meteorPrefab.transform.rotation);  
				break;
            case DisasterType.Earthquake:
				Instantiate(this.earthquakePrefab, Vector3.zero, Quaternion.identity);
                break;
        }

		if (this.selectedDisaster != DisasterType.None)
		{
			this.disasterCount--;
			this.StartCoroutine(this.CheckWinConditions());
		}

		this.selectedDisaster = DisasterType.None;
    }

	/// <summary>
	/// Win condition coroutine.
	/// </summary>
	/// <returns>Coroutine iterator.</returns>
	internal IEnumerator CheckWinConditions()
	{
		yield return new WaitForSeconds(5f);

		if (this.populationManager.casualties > this.casualtyTarget)
		{
			this.disasterCount = 0;
			this.headlineLabel.text = "Unnatural!" + Environment.NewLine + "You destroyed " + this.populationManager.casualties;
			this.audio.Play();
			this.continueButton.SetActive(true);
		} 
		else if (this.disasterCount <= 0)
		{
			this.headlineLabel.text = "What a disaster!" + Environment.NewLine + "Only " + this.populationManager.casualties + " casualties";
			this.continueButton.SetActive(true);
		}
	}
}


