﻿// --------------------------------
// <copyright file="Earthquake.cs" company="COMPANY">
//     Copyright (C) COMPANY.  All rights reserved.
// </copyright>
// --------------------------------

using System.Collections;
using UnityEngine;

/// <summary>
/// Earthquake class.
/// </summary>
public class Earthquake : MonoBehaviour
{
	public float duration = 5f;

	public float defaultStrength = 0.002f;

	public int maxIncrements = 5;

	public int increments = 1;

	private RigidbodyShaker groundShaker;

	/// <summary>
	/// Input manager.
	/// </summary>
	private InputManager input;
	
    /// <summary>
    /// Initialize script state.
    /// </summary>
    internal void Start()
    {
		this.groundShaker = FindObjectOfType<RigidbodyShaker>();

		this.groundShaker.strength = this.defaultStrength;
		this.groundShaker.isActive = true;

		// shake camera
		var cam = FindObjectOfType<CameraOrbit>();
		if (cam)
		{
			cam.isActive = true;
		}

		this.StartCoroutine(this.Kill(this.duration));
    }

	/// <summary>
	/// Called when script is enabled.
	/// </summary>
	internal void OnEnable()
	{
		if (!this.input)
		{
			this.input = FindObjectOfType<InputManager>();
		}
		
		this.input.PickEvent += this.HandlePickEvent;
	}
	
	/// <summary>
	/// Called when script is disabled.
	/// </summary>
	internal void OnDisable()
	{
		if (this.input)
		{
			this.input.PickEvent -= this.HandlePickEvent;
		}
	}

	private void HandlePickEvent(Vector3 point)
	{
		if (this.increments < this.maxIncrements)
		{
			this.increments++;
		}

		this.groundShaker.strength = this.defaultStrength * this.increments;
	}
	
	/// <summary>
	/// Kill this after particles expire.
	/// </summary>
	/// <returns>Coroutine iterator.</returns>
	internal IEnumerator Kill(float lifespan)
	{
		yield return new WaitForSeconds(lifespan);
		
		this.groundShaker.isActive = false;

		// unshake camera
		var cam = FindObjectOfType<CameraOrbit>();
		if (cam)
		{
			cam.isActive = false;
		}

		Destroy(this.gameObject);
	}
}