﻿// --------------------------------
// <copyright file="InputManager.cs" company="COMPANY">
//     Copyright (C) COMPANY.  All rights reserved.
// </copyright>
// --------------------------------

using System;
using UnityEngine;

/// <summary>
/// InputManager class.
/// </summary>
public class InputManager : MonoBehaviour
{
	/// <summary>
	/// UI camera.
	/// </summary>
	public UICamera uiCamera;

    /// <summary>
    /// Event to fire when game ends.
    /// </summary>
    public event Action<Vector3> PickEvent;

	/// <summary>
	/// Initialize script state.
	/// </summary>
	internal void Start()
	{
		UICamera.fallThrough = this.gameObject;
	}

	/// <summary>
	/// Handles a click event.
	/// </summary>
	internal void OnClick()
	{
		Debug.Log("clicked " + UICamera.lastTouchPosition);
		var ray = Camera.main.ScreenPointToRay(UICamera.lastTouchPosition);
		RaycastHit hit;
		
		if (Physics.Raycast(ray, out hit, 100.0f))
		{
			if (this.PickEvent != null)
			{
				this.PickEvent(hit.point);
			}
		}
	}
}