﻿// --------------------------------
// <copyright file="ParticleManager.cs" company="Rumor Games">
//     Copyright (C) Rumor Games, LLC.  All rights reserved.
// </copyright>
// --------------------------------

using UnityEngine;
using System.Collections;

/// <summary>
/// ParticleManager class.
/// </summary>
public class ParticleManager : MonoBehaviour
{
    /// <summary>
    /// Whether the particle system has been dismissed.
    /// </summary>
    protected bool isDismissed;

    /// <summary>
    /// Stop particle emitter/audio and reparent.
    /// </summary>
    public void Dismiss()
    {
        if (!this.isDismissed)
        {
            this.particleSystem.enableEmission = false;
            if (this.gameObject.audio)
            {
                this.gameObject.audio.Stop();
            }

            if (this.transform.parent)
            {
                this.transform.parent = this.transform.parent.parent;
            }

            this.StopAllCoroutines();

            this.StartCoroutine(this.DestroyEmitter());

            this.isDismissed = true;
        }
    }

    /// <summary>
    /// Destroy emitter coroutine.
    /// </summary>
    /// <returns>Coroutine iterator.</returns>
    internal IEnumerator DestroyEmitter()
    {
        yield return new WaitForSeconds(this.particleSystem.startLifetime);

        Destroy(this.gameObject);
    }
}